#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <arpa/inet.h>

#include <libnetfilter_conntrack/libnetfilter_conntrack.h>
#include <libnetfilter_conntrack/libnetfilter_conntrack_tcp.h>

/* 
 * TCP 
 */
int tcp_ss  = 0; /* SYN_SENT */
int tcp_sr  = 0; /* SYN_RECV */
int tcp_e   = 0; /* ESTABLISHED */
int tcp_fw  = 0; /* FIN_WAIT */
int tcp_cw  = 0; /* CLOSE_WAIT */
int tcp_la  = 0; /* LAST_ACK */
int tcp_tw  = 0; /* TIME_WAIT */
int tcp_c   = 0; /* CLOSE */

int tcp_u   = 0; /* UNREPLIED */
int tcp_ha  = 0; /* HALF_ASSURED */
int tcp_a   = 0; /* ASSURED */
int tcp_tot = 0; /* total TCP */

/* 
 * UDP 
 */
int udp_u   = 0; /* UNREPLIED */
int udp_ha  = 0; /* HALF_ASSURED */
int udp_a   = 0; /* ASSURED */
int udp_tot = 0; /* total UDP */

/* 
 * ICMP 
 */
int icmp_u   = 0; /* UNREPLIED */
int icmp_ha  = 0; /* HALF_ASSURED */
int icmp_tot = 0; /* total ICMP */

/* 
 * IGMP
 */
int igmp_u   = 0; /* UNREPLIED */
int igmp_ha  = 0; /* HALF_ASSURED */
int igmp_tot = 0; /* total IGMP */

/* 
 * other
 */
int other_u   = 0; /* UNREPLIED */
int other_ha  = 0; /* HALF_ASSURED */
int other_a   = 0; /* ASSURED */
int other_tot = 0; /* total other */

/* 
 * total
 */
int total_u   = 0; /* UNREPLIED */
int total_ha  = 0; /* HALF_ASSURED */
int total_a   = 0; /* ASSURED */
int total     = 0; /* total total */

static int event_cb(enum nf_conntrack_msg_type type,
		    struct nf_conntrack *ct,
		    void *data)
{
	u_int32_t conn_state=nfct_get_attr_u32(ct,ATTR_STATUS);	
	++total;
	if (conn_state & IPS_ASSURED) ++total_a;
	else if (conn_state & IPS_SEEN_REPLY) ++total_ha;
	else ++total_u;
	switch (nfct_get_attr_u8(ct,ATTR_L4PROTO))
	{
	case IPPROTO_TCP:
	    switch(nfct_get_attr_u8(ct,ATTR_TCP_STATE))
	    {
	    case TCP_CONNTRACK_SYN_SENT:
		++tcp_ss;
		break;
	    case TCP_CONNTRACK_SYN_RECV:
		++tcp_sr;
		break;
	    case TCP_CONNTRACK_ESTABLISHED:
		++tcp_e;
		break;
	    case TCP_CONNTRACK_FIN_WAIT:
		++tcp_fw;
		break;
	    case TCP_CONNTRACK_CLOSE_WAIT:
		++tcp_cw;
		break;
	    case TCP_CONNTRACK_LAST_ACK:
		++tcp_la;
		break;
	    case TCP_CONNTRACK_TIME_WAIT:
		++tcp_tw;
		break;
	    case TCP_CONNTRACK_CLOSE:
		++tcp_c;
		break;
	    }
	    ++tcp_tot;
	    if (conn_state & IPS_ASSURED) ++tcp_a;
	    else if (conn_state & IPS_SEEN_REPLY) ++tcp_ha;
	    else ++tcp_u;
	    break;
	case IPPROTO_UDP:
	    ++udp_tot;
	    if (conn_state & IPS_ASSURED) ++udp_a;
	    else if (conn_state & IPS_SEEN_REPLY) ++udp_ha;
	    else ++udp_u;
	    break;
	case IPPROTO_ICMP:
	    ++icmp_tot;
	    if (conn_state & IPS_SEEN_REPLY) ++icmp_ha;
	    else ++icmp_u;
	    break;
	case IPPROTO_IGMP:
	    ++igmp_tot;
	    if (conn_state & IPS_SEEN_REPLY) ++igmp_ha;
	    else ++igmp_u;
	    break;
	default:
	    ++other_tot;
	    if (conn_state & IPS_ASSURED) ++other_a;
	    else if (conn_state & IPS_SEEN_REPLY) ++other_ha;
	    else ++other_u;
	}
	return NFCT_CB_CONTINUE;
}

int main(void)
{
	int ret;
	u_int32_t family = AF_INET;
	struct nfct_handle *h;

	h = nfct_open(CONNTRACK, NF_NETLINK_CONNTRACK_NEW | 
				 NF_NETLINK_CONNTRACK_UPDATE);
	if (!h) {
		perror("Cannot open connection to libnetfilter_conntrack");
		return 1;
	}
	nfct_callback_register(h, NFCT_T_ALL, event_cb, NULL);
	ret = nfct_query(h, NFCT_Q_DUMP, &family);
	nfct_close(h);
	if (ret == -1)
	{
		printf("(%d)(%s)\n", ret, strerror(errno));
		return 1;
	}
	printf ("tcp_ss:%d tcp_sr:%d tcp_e:%d tcp_fw:%d tcp_cw:%d tcp_la:%d tcp_tw:%d tcp_c:%d tcp_a:%d tcp_u:%d tcp_ha:%d tcp_tot:%d ", tcp_ss, tcp_sr, tcp_e, tcp_fw, tcp_cw, tcp_la, tcp_tw, tcp_c, tcp_a, tcp_u, tcp_ha, tcp_tot);
	printf ("udp_a:%d udp_u:%d udp_ha:%d udp_tot:%d ", udp_a, udp_u, udp_ha, udp_tot);
	printf ("icmp_u:%d icmp_ha:%d icmp_tot:%d ", icmp_u, icmp_ha, icmp_tot);
	printf ("igmp_u:%d igmp_ha:%d igmp_tot:%d ", igmp_u, igmp_ha, igmp_tot);
	printf ("other_a:%d other_u:%d other_ha:%d other_tot:%d ", other_a, other_u, other_ha, other_tot);
	printf ("tot_a:%d tot_u:%d tot_ha:%d tot:%d\n", total_a, total_u, total_ha, total);
	return 0;
}
